/* $Id: userauth.C,v 1.13 1999/03/24 07:25:31 dm Exp $ */

/*
 *
 * Copyright (C) 1999 David Mazieres (dm@uun.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 */

#include "sfsrwcd.h"

userauth::userauth (sfs_aid a, const ref<server> &s)
  : aid (a), sp (s), tmo (NULL), cbase (NULL), aborted (false), ntries (0)
{
  // XXX - NFS3ERR_JUKEBOX panics FreeBSD
  if (!uvfs)
    tmo = timecb (time (NULL) + 5, wrap (this, &userauth::timeout));
  sendreq ();
}

userauth::~userauth ()
{
  abort ();
}

void
userauth::timeout ()
{
  tmo = NULL;
  while (!sbpvec.empty ())
    nfs3_err (sbpvec.pop_front (), NFS3ERR_JUKEBOX);
}

void
userauth::pushreq (svccb *sbp)
{
  if (tmo || uvfs)
    sbpvec.push_back (sbp);
  else
    nfs3_err (sbp, NFS3ERR_JUKEBOX);
}

void
userauth::abort ()
{
  aborted = true;
  if (tmo) {
    timecb_remove (tmo);
    tmo = NULL;
  }
  if (cbase) {
    cbase->cancel ();
    cbase = NULL;
  }
  while (!sbpvec.empty ())
    sp->dispatch (sbpvec.pop_front ());
}

void
userauth::sendreq ()
{
  if (!ntries)
    seqno = ++sp->seqno;
  sfscd_agentreq_arg arg;
  arg.aid = aid;
  arg.agentreq.set_type (AGENTCB_AUTHINIT);
  arg.agentreq.init->ntries = ntries;
  arg.agentreq.init->requestor = "";
  arg.agentreq.init->authinfo = sp->authinfo;
  arg.agentreq.init->seqno = seqno;

  cbase = cdc->call (SFSCDCBPROC_AGENTREQ, &arg, &ares,
		     wrap (this, &userauth::aresult));
}

void
userauth::aresult (clnt_stat err)
{
  cbase = NULL;
  if (err) {
    warn << "userauth: " << err << "\n";
    finish (0);
  }
  else if (!ares.authenticate)
    finish (0);
  else {
    sfs_loginarg arg;
    arg.seqno = seqno;
    arg.certificate = *ares.certificate;
    sp->sfsc->call (SFSPROC_LOGIN, &arg, &sres,
		    wrap (mkref (this), &userauth::sresult));
  }
}

void
userauth::sresult (clnt_stat err)
{
  cbase = NULL;
  if (err)
    finish (0);
  else if (aborted) {
    if (sres.status == SFSLOGIN_OK && sp->sfsc)
      sp->sfsc->call (SFSPROC_LOGOUT, sres.authno.addr (), NULL,
		      aclnt_cb_null);
  }
  else
    switch (sres.status) {
    case SFSLOGIN_OK:
      finish (*sres.authno);
      break;
    case SFSLOGIN_MORE:
      {
	sfscd_agentreq_arg arg;
	arg.aid = aid;
	arg.agentreq.set_type (AGENTCB_AUTHMORE);
	arg.agentreq.more->authinfo = sp->authinfo;
	arg.agentreq.more->seqno = seqno;
	arg.agentreq.more->challenge = *sres.resmore;
	cbase = cdc->call (SFSCDCBPROC_AGENTREQ, &arg, &ares,
			   wrap (this, &userauth::aresult));
	break;
      }
    case SFSLOGIN_BAD:
      ntries++;
      sendreq ();
      break;
    case SFSLOGIN_ALLBAD:
      finish (0);
      break;
    default:
      warn << "userauth: bad status in loginres!\n";
      finish (0);
    }
}

void
userauth::finish (u_int32_t authno)
{
  sp->mapuser (aid, authno);
}
